#include "framecpp/config.h"

#include "framecpp/Common/Description.hh"
#include "framecpp/Common/IOStream.hh"

#include "framecpp/Version6/FrameSpec.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"

using namespace FrameCPP::Version_6;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;

//=======================================================================
//=======================================================================
FrTable::
FrTable( )
  : Object( CLASS_ID, StructDescription( ) )
{
}

FrTable::
FrTable( const FrTable& Table )
  : Object( CLASS_ID, StructDescription( ) ),
    FrTableNPS( Table )
{
}

FrTable::
FrTable( const std::string& name, INT_4U nrows )
  : Object( CLASS_ID, StructDescription( ) )
{
  setName( name );
}

FrTable::
FrTable( const std::string& Name,
	 const std::string& Comment,
	 INT_4U NRows,
	 column_name_container_type ColumnNames )
  : Object( CLASS_ID, StructDescription( ) )
{
  setName( Name );
  AppendComment( Comment );
}

FrTable::
FrTable( const FrTableNPS& Source )
  :  Object( CLASS_ID, StructDescription( ) ),
     FrTableNPS( Source )
{
}

FrTable::
FrTable( IStream& Stream )
  : Object( CLASS_ID, StructDescription( ) )
{
  m_data( Stream );
  m_refs( Stream );

  Stream.Next( this );
}

FrTable::
FrTable( Previous::FrTable& Source, IStream* Stream )
  : Object( CLASS_ID, StructDescription( ) ),
    FrTableNPS( Source )
{
  if ( Stream )
  {
    //-------------------------------------------------------------------
    // Fix references
    //-------------------------------------------------------------------
    Stream->ReplaceRef( Source.RefColumn( ),
			&(RefColumn( )), m_refs.MAX_REF );

  }
}

FrTable::
~FrTable( )
{
}

FrameCPP::cmn_streamsize_type  FrTable::
Bytes( const Common::StreamBase& Stream ) const
{
  cmn_streamsize_type retval
    = m_data.Bytes( Stream )
    + m_refs.Bytes( Stream )
    + Stream.PtrStructBytes( )	// next
    ;
  return retval;
}

FrTable* FrTable::
Create( IStream& Stream ) const
{
  return new FrTable( Stream );
}

FrTable& FrTable::
Merge( const FrTable& RHS )
{
  throw
    Unimplemented( "FrTable& FrTable::Merge( const FrTable& RHS )",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
  return *this;
}

const char* const FrTable::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrTable::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( StructName( ), CLASS_ID, "Table Data Structure" ) );
	
    data_type::Describe< FrSE >( ret );
    refs_type::Describe< FrSE >( ret );

    ret( FrSE( "next", ptr_struct_type::Desc( StructName( ) ) ) );
  }

  return &ret;
}

void FrTable::
Write( OStream& Stream ) const
{
  m_data( Stream );
  m_refs( Stream );

  WriteNext( Stream );
}

const std::string& FrTable::
GetNameSlow( ) const
{
  return GetName( );
}

bool FrTable::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return compare( *this, Obj );
}

FrTable& FrTable::
operator+=( const FrTable& RHS )
{
  throw Unimplemented( "FrTable& FrTable::operator+=( const FrTable& RHS )",
		       DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrTable::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    //-------------------------------------------------------------------
    // Same version
    //-------------------------------------------------------------------
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrTable* >( this ) );
  }
  try
  {
    //-------------------------------------------------------------------
    // Copy non-reference information
    //-------------------------------------------------------------------
    // Do actual down conversion
    std::auto_ptr< Previous::FrTable >
      prev( new Previous::FrTable( GetName( ),
				   GetNRow( ) ) );
    if ( prev.get( ) )
    {
      prev->AppendComment( GetComment( ) );
    }
    if ( Stream )
    {
      Stream->ReplaceRef( prev->RefColumn( ), &(RefColumn( )), MAX_REF );
    }
    //-------------------------------------------------------------------
    // Return demoted object
    //-------------------------------------------------------------------
    std::auto_ptr< FrameSpec::Object > retval( prev.get( ) );
    if ( retval.get( ) )
    {
      prev.release( );
    }
    return retval;
  }
  catch( ... )
  {
  }
  throw
    Unimplemented( "Object* FrTable::Demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrTable::
promote( INT_2U Source, Object* Obj, IStream* Stream ) const
{
  return PromoteObject< Previous::FrTable, FrTable >
    ( DATA_FORMAT_VERSION, Source, Obj, Stream );
}
